"""
    refinitiv-data is a Python library to access Refinitiv Data Platform with Python.
"""

# _version.py file is generated by setuptools_scm on build
try:
    from ._version import __version__
except ImportError:
    __version__ = "0.0.0"

from typing import TYPE_CHECKING as _TYPE_CHECKING

from ._tools import lazy_attach as _lazy_attach

if _TYPE_CHECKING:
    from ._configure import get_config, load_config
    from ._open_state import OpenState
    from . import eikon, delivery, session, content, errors, usage_collection, discovery
    from ._access_layer import (
        dates_and_calendars,
        news,
        open_session,
        close_session,
        get_data,
        get_history,
        PricingStream,
        open_pricing_stream,
    )

import warnings as _warnings

_warnings.formatwarning = (
    lambda message, category, filename, lineno, line=None: f"{filename}:{lineno}:{category.__name__}: {message}\n"
)

_submodules = {
    "content",
    "delivery",
    "discovery",
    "eikon",
    "errors",
    "session",
    "usage_collection",
}

_submod_attrs = {
    "_configure": ["get_config", "load_config"],
    "_open_state": ["OpenState"],
    "_access_layer": [
        "dates_and_calendars",
        "news",
        "open_session",
        "close_session",
        "get_data",
        "get_history",
        "PricingStream",
        "open_pricing_stream",
    ],
}

__getattr__, __dir__, __all__ = _lazy_attach(__name__, submodules=_submodules, submod_attrs=_submod_attrs)
